@@ -8,19 +8,19 @@ from fastapi.openapi.utils import get_openapi
 SchemaMutator = Callable[[dict], dict]
 
 
-def apply_mutators(app: FastAPI, *mutators: SchemaMutator) -> None:
+def apply_mutators(app: FastAPI, *mutators):
     previous = getattr(app, "openapi", None)
 
-    def _openapi():
-        base = (
+    def patched():
+        base_schema = (
             previous()
             if callable(previous)
             else get_openapi(title=app.title, version=app.version, routes=app.routes)
         )
-        schema = base
+        schema = dict(base_schema)
         for m in mutators:
             schema = m(schema)
         app.openapi_schema = schema
         return schema
 
-    app.openapi = _openapi
+    app.openapi = patched
